#include <stdio.h>
#include "s_alloc.h"

void *smalloc(size_t size)
{
	void *p = malloc((size_t)(sizeof(struct alloc_info) + size));

	if (p == NULL)
	{
		perror("Error while allocating memory");
		fprintf(stderr, "Tried to allocate %u bytes\n", size);
		fflush(stderr);
		exit(1);
	}

	/* Memory allocation succeeded */
	struct alloc_info *block = p;
	block->size = size;
	block->ptr = p + sizeof(struct alloc_info);

	mem_s.last_al = block->size;
	mem_s.curr_act += block->size;
	if (block->size > mem_s.max_al)
	{
		mem_s.max_al = block->size;
	}
	if (mem_s.curr_act > mem_s.max_act)
	{
		mem_s.max_act = mem_s.curr_act;
	}
	mem_s.total_al += block->size;

	return block->ptr;
}

void sfree(void *ptr)
{
	if (ptr == NULL)
	{
		return;
	}
	struct alloc_info *block = ptr - sizeof(struct alloc_info);
	mem_s.curr_act -= block->size;
	free(block);
}

void *srealloc(void *ptr, size_t size)
{
	if (ptr == NULL)
	{
		return smalloc(size);
	}

	size_t size_old = getmemsz(ptr);
	int64_t size_diff = (int64_t)size - size_old;

	void *new_ptr = realloc(ptr - sizeof(struct alloc_info), (size_t)(sizeof(struct alloc_info) + size));

	/* If we can't allocate enough memory, we shall return old pointer */
	if (new_ptr == NULL)
	{
		return ptr;
	}

	struct alloc_info *block = new_ptr;
	block->size = size;
	block->ptr = new_ptr + sizeof(struct alloc_info);

	mem_s.last_al = block->size;
	mem_s.curr_act += size_diff;
	if (block->size > mem_s.max_al)
	{
		mem_s.max_al = block->size;
	}
	if (mem_s.curr_act > mem_s.max_act)
	{
		mem_s.max_act = mem_s.curr_act;
	}
	mem_s.total_al += block->size;

	return block->ptr;
}

size_t getmemsz(void *ptr)
{
	void *p = ptr - sizeof(struct alloc_info);
	struct alloc_info *block = p;
	return block->size;
}

void print_stats(void)
{
	fprintf(stderr, "Last allocation: %u bytes\n"
		"Currently active: %llu bytes\n"
		"Biggest allocation: %u bytes\n"
		"Maximum memory usage: %llu bytes\n"
		"Allocated in total: %llu bytes\n\n",
		mem_s.last_al, mem_s.curr_act, mem_s.max_al,
		mem_s.max_act, mem_s.total_al);
	fflush(stderr);
}
