#ifndef S_ALLOC_H
#define S_ALLOC_H

/************************************************
*  s_alloc.h
*  Memory allocation with statistics
************************************************/

#include <stdlib.h>
#include <stdint.h>

/* Stores allocated bytes and pointer for each malloc()'ed item */
struct alloc_info
{
	size_t		size;
	void		*ptr;
};

/* Stores overall statistics of memory allocations */
struct mem_stats
{
	size_t		last_al;	/* Last allocation */
	uint64_t	curr_act;	/* Currently allocated bytes */
	size_t		max_al;		/* Max. allocated bytes by one malloc() call */
	uint64_t	max_act;	/* Max. active bytes */
	uint64_t	total_al;	/* Total number of bytes allocated in this program */
};
struct mem_stats mem_s;


/* malloc*/
void *smalloc(size_t);

/* free */
void sfree(void *);

/* realloc */
void *srealloc(void *, size_t);


/* Gets number of bytes allocated for a block of memory, which ptr is pointing at */
size_t getmemsz(void *);

/* Prints current memory statistics to stderr */
void print_stats(void);

#endif /* S_ALLOC_H */